package mmkurssit;

import java.io.File;
import java.util.List;

/**
 * CRC sisällöt tähän
 * @author aichi
 * @version 26 Feb 2025
 *
 */
public class MMKurssit {

    private Kurssit kurssit = new Kurssit();
    private Tapaamiset tapaamiset = new Tapaamiset();
    
//==================================================================

    
    /**
     * Lisätään uusi kurssi
     * @param esimk lisättävä kurssi
     * @throws SailoException jos lisääminen ei onnistu
     */
    public void lisaa(Kurssi esimk) throws SailoException {
        kurssit.lisaa(esimk);
    }
    
    
    
    /**
     * Lisätään uusi tapaaminen
     * @param tap lisättävä tapaaminen
     */
    public void lisaa(Tapaaminen tap) {
        tapaamiset.lisaa(tap);
    }
    
    /**
     * @return kurssien kpl
     */
    public int getKurssienKpl() {
        return kurssit.getKpl();
    }
    
    
    /**
     * @param i kyseisen kurssin indeksi
     * @return pyydetty kurssi kurssit-taulkosta
     * @example
     * <pre name="test">
     * #import java.util.List;
     *   Tapaamiset test = new Tapaamiset();
     *   Tapaaminen t1 = new Tapaaminen(1);
     *   t1.rekisteroi();
     *   t1.tayta();
     *   test.lisaa(t1);
     *   Tapaaminen t2 = new Tapaaminen(1);
     *   t2.rekisteroi();
     *   t2.tayta();
     *   test.lisaa(t2);
     *   Tapaaminen t3 = new Tapaaminen(3);
     *   t3.rekisteroi();
     *   t3.tayta();
     *   test.lisaa(t3);
     *   Tapaaminen t4 = new Tapaaminen(1);
     *   t4.rekisteroi();
     *   t4.tayta();
     *   test.lisaa(t4);
     *   
     *   List<Tapaaminen> tulos = test.annaTapaamiset(1);
     *     tulos.size() === 3;
     *   tulos = test.annaTapaamiset(2);
     *     tulos.size() === 0;
     *   tulos = test.annaTapaamiset(3);
     *     tulos.size() === 1; 
     * </pre>
     */
    public Kurssi annaKurssi(int i) {
        return kurssit.anna(i);
    }
    
    
    /**
     * @return Kurssit-olio
     */
    public Kurssit kaikkiKurssit() {
        return this.kurssit;
    }
    
    /**
     * @param i indeksi
     * @return listalta indeksi paikalla oleva tapaaminen-olio
     */
    public Tapaaminen annaTapaaminen(int i) {
        return tapaamiset.annaTapaaminen(i);
    }
    
    /**
     * @param kurssiID kyseinen kurssi
     * @return tapaamiset (laitettuna listaan) jotka liittyvät kyseiseen kurssiin
     */
    public List<Tapaaminen> annaTapaamiset(int kurssiID){
        return tapaamiset.annaTapaamiset(kurssiID);
    }
    
    
    /**
     * Lukee kurssien tiedot tiedostosta
     * @param nimi mmkurssin hakemiston nimi
     * @throws SailoException jos lukeminen epäonnistuu
     */
    public void lueTiedostosta(String nimi) throws SailoException {
        
        File kohde = new File(nimi);//varmistetaan nimi-nimellä on hakemisto
        kohde.mkdir();
        
        kurssit = new Kurssit(); //helppo tapa tyhjentää vanha 
        tapaamiset = new Tapaamiset();
        
        kurssit.lueTiedostosta(nimi);
        tapaamiset.lueTiedostosta(nimi);
    }
    
    
    /**
     * Lukee tapaamisten tiedot tiedostosta
     * @param hakemisto mmkurssin hakemiston nimi
     * @throws SailoException jos tallentaminen epäonnistuu
     */
    public void tallenna(String hakemisto) throws SailoException {
        String virhe = "";
        try {
            kurssit.tallenna(hakemisto);
        } catch (SailoException ex) {
            virhe = ex.getMessage();
        }
        
        try {
            tapaamiset.tallenna(hakemisto);
        } catch (SailoException ex) {
            virhe += ex.getMessage();
        }
        
        if ( !"".equals(virhe) ) throw new SailoException(virhe);
    }
    
//===================================================================
    
    /**
     * @param args ei käytössä
     */
    public static void main(String[] args) {
    
            MMKurssit mmkurssit = new MMKurssit();
            
            try {
                mmkurssit.lueTiedostosta("tiedot");
            } catch (SailoException e) {
                System.err.println("Lukeminen ei onnistunut.");
            }
            
            Kurssi japani4 = new Kurssi();
            japani4.rekisteroi();
            japani4.tayta();
            
            Kurssi ohjelmointi2 = new Kurssi();
            ohjelmointi2.rekisteroi();
            ohjelmointi2.tayta();
            
            try {
                mmkurssit.lisaa(japani4);
                mmkurssit.lisaa(ohjelmointi2);
                //ohjelma.lisaa(japani4);
                //ohjelma.lisaa(ohjelmointi2);
                //ohjelma.lisaa(japani4);
                //ohjelma.lisaa(ohjelmointi2);//heittää error
            } catch (SailoException e) {
                System.err.println(e.getMessage());
            }
            
            
            for (int i=0; i<mmkurssit.getKurssienKpl(); i++) {
                Kurssi kurssi = mmkurssit.annaKurssi(i);
                kurssi.tulosta(System.out);
            }
            
            
            System.out.println("======================================");
            
            Tapaaminen j4_1 = new Tapaaminen(japani4.getID());
            j4_1.rekisteroi();
            j4_1.tayta();
            //j4_1.tulosta(System.out);
            Tapaaminen ohj2_1 = new Tapaaminen(ohjelmointi2.getID());
            ohj2_1.rekisteroi();
            ohj2_1.tayta();
            Tapaaminen j4_2 = new Tapaaminen(japani4.getID());
            j4_2.rekisteroi();
            j4_2.tayta();
            
            mmkurssit.lisaa(j4_1);
            mmkurssit.lisaa(ohj2_1);
            mmkurssit.lisaa(j4_2);
            
            List<Tapaaminen> j4_tap = mmkurssit.annaTapaamiset(japani4.getID());
            for(Tapaaminen tap : j4_tap) {
                tap.tulosta(System.out);
            }
            
            try {
                mmkurssit.tallenna("tiedot");
            } catch (SailoException e) {
                System.err.println("Tallentaminen ei onnistunut");
            }
    
    }


}
