package mmkurssit;

import java.io.*;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Scanner;


/**
 * Tietorakenne (tällä kerralla taulukko), jossa säilytetään tapaamiset alkioina.
 * @author aichi
 * @version 25 Feb 2025
 *
 */
public class Tapaamiset {

    private Collection<Tapaaminen> alkiot = new ArrayList<Tapaaminen>();
    //private String folderName = "";
    
//======================================================
    
    
    /** Alustaminen */
    public Tapaamiset() {
        //
    }
    
    /** 
     *@param tap lisattava tapaaminen 
     */
    public void lisaa(Tapaaminen tap) {
        alkiot.add(tap);
        
    }
    
    
    /**
     * @param kurssiID kyseisen kurssin ID
     * @return Lisa, jossa on tapaamisia, jonka kID on sama kuin parametriksi annettu kurssiID
     * @example
     * <pre name="test">
     * #import java.util.List;
     *   Tapaamiset test = new Tapaamiset();
     *   Tapaaminen t1 = new Tapaaminen(1);
     *   t1.rekisteroi();
     *   t1.tayta();
     *   test.lisaa(t1);
     *   Tapaaminen t2 = new Tapaaminen(1);
     *   t2.rekisteroi();
     *   t2.tayta();
     *   test.lisaa(t2);
     *   Tapaaminen t3 = new Tapaaminen(3);
     *   t3.rekisteroi();
     *   t3.tayta();
     *   test.lisaa(t3);
     *   Tapaaminen t4 = new Tapaaminen(1);
     *   t4.rekisteroi();
     *   t4.tayta();
     *   test.lisaa(t4);
     *   
     *   List<Tapaaminen> tulos = test.annaTapaamiset(1);
     *     tulos.size() === 3;
     *   tulos = test.annaTapaamiset(2);
     *     tulos.size() === 0;
     *   tulos = test.annaTapaamiset(3);
     *     tulos.size() === 1;
     *   
     * </pre>
     */
    public List<Tapaaminen> annaTapaamiset(int kurssiID) {
        
        List<Tapaaminen> loydetyt = new ArrayList<Tapaaminen>();
        for (Tapaaminen tap : alkiot) {
            if (tap.getkID() == kurssiID) loydetyt.add(tap);
        }
        return loydetyt;
    }
    
    
    /**
     * @param indeksi indeksi
     * @return lsitalta i-paikalla oleva tapaaminen olio
     */
    public Tapaaminen annaTapaaminen(int indeksi) {
        return ((ArrayList<Tapaaminen>) alkiot).get(indeksi);
    }

    
    /**
     * @param folderName luettava tiedoston nimi
     * @throws SailoException jos ongelmia tiedostojen kanssa
     * 
     */
    public void lueTiedostosta(String folderName) throws SailoException {
        String nimi = folderName + "/tapaamiset.dat";
        File ftied = new File(nimi);
        
        try(Scanner fi = new Scanner(new FileInputStream(ftied))){
            while(fi.hasNextLine()) {
                String rivi = fi.nextLine().trim();
                if ("".equals(rivi) || rivi.charAt(0) == ';' ) continue;
                Tapaaminen tap = new Tapaaminen();
                tap.parse(rivi);
                tap.setRadNro();
                lisaa(tap);
            }
        } catch (FileNotFoundException ex) {
            throw new SailoException("Ei saa luettua tiedostoa " + nimi);
        }
    }
    
    
    /**
     * Tallentaa tapaamiset tiesdostoon.
     * @param folderName mihin hakemistoon
     * @throws SailoException jos tallennus epäonnistu
     */
    public void tallenna(String folderName) throws SailoException {
        File ftied = new File(folderName + "/tapaamiset.dat");
        
        try (PrintStream fo = new PrintStream(new FileOutputStream(ftied, false))){
            
            for (int i=0; i<this.alkiot.size(); i++) {
                Tapaaminen tap = ((ArrayList<Tapaaminen>) this.alkiot).get(i);
                fo.println(tap.toString());
            }
            
        } catch (FileNotFoundException e) {
            throw new SailoException("Tiedosto"+ftied.getAbsolutePath() + " ei löydy");
        }
    }
    
//=======================================================

    /**
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        
        Tapaamiset tapaamiset = new Tapaamiset();
        
        try {
            tapaamiset.lueTiedostosta("tiedot");
        } catch (SailoException ex) {
            System.err.println(ex.getMessage());
        }
        
        Tapaaminen j4_1 = new Tapaaminen(1);
        j4_1.rekisteroi();
        j4_1.tayta();
        
        Tapaaminen ohj2_1 = new Tapaaminen(2);
        ohj2_1.rekisteroi();
        ohj2_1.tayta();
        
        /*Tapaaminen j4_2 = new Tapaaminen(1);
        j4_2.rekisteroi();
        j4_2.tayta();*/
        
        
        tapaamiset.lisaa(j4_1);
        tapaamiset.lisaa(ohj2_1);
        //tapaamiset.lisaa(j4_2);
        
        System.out.println("============ Tapaamiset testi ==================");
        
        System.out.println(j4_1);
        
        List<Tapaaminen> tapaamiset1 = tapaamiset.annaTapaamiset(1);
        //"var" käytetään, kun operaattorin oikealla puolella oleva palauttaa selkeästi yksi tyyppi
        // sitten koodin kääntäessä kääntäjä automaattisesti kirjoittaa tyypin nimen var-paikalle.
        // Eli täähän voisi kirjoittaa var tapaamiset2 = ...;
        
        for(Tapaaminen tap : tapaamiset1) {
            System.out.println(tap.getkID()+" ");
            tap.tulosta(System.out);
        }
        
        
        try {
            tapaamiset.tallenna("tiedot");
        } catch (SailoException e) {
            System.err.println(e.getMessage());
        }
    
    }


}
